use std::io::{self, Write};

/// Wrapper around the Write trait that counts how many bytes have been written in total.
/// Heavily inspired by https://stackoverflow.com/a/42189386
pub struct CountingWrite<W> {
    inner: W,
    count: usize,
}

impl<W> CountingWrite<W>
where
    W: Write,
{
    pub fn new(writer: W) -> Self {
        Self {
            inner: writer,
            count: 0,
        }
    }

    pub fn bytes_written(&self) -> usize {
        self.count
    }
}

impl<W> Write for CountingWrite<W>
where
    W: Write,
{
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        let res = self.inner.write(buf);

        if let Ok(count) = res {
            self.count += count;
        }

        res
    }

    fn flush(&mut self) -> io::Result<()> {
        self.inner.flush()
    }
}
